@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2004-2005 William Jon McCann <mccann@jhu.edu>
+ * Copyright (C) 2004-2006 William Jon McCann <mccann@jhu.edu>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -47,33 +47,37 @@ static gboolean do_lock       = FALSE;
 static gboolean do_cycle      = FALSE;
 static gboolean do_activate   = FALSE;
 static gboolean do_deactivate = FALSE;
-static gboolean do_throttle   = FALSE;
-static gboolean do_unthrottle = FALSE;
 static gboolean do_version    = FALSE;
 static gboolean do_poke       = FALSE;
+static gboolean do_inhibit    = FALSE;
 
 static gboolean do_query      = FALSE;
 
+static char    *inhibit_reason      = NULL;
+static char    *inhibit_application = NULL;
+
 static GOptionEntry entries [] = {
         { "exit", 0, 0, G_OPTION_ARG_NONE, &do_quit,
           N_("Causes the screensaver to exit gracefully"), NULL },
-        { "query", 0, 0, G_OPTION_ARG_NONE, &do_query,
+        { "query", 'q', 0, G_OPTION_ARG_NONE, &do_query,
           N_("Query the state of the screensaver"), NULL },
-        { "lock", 0, 0, G_OPTION_ARG_NONE, &do_lock,
+        { "lock", 'l', 0, G_OPTION_ARG_NONE, &do_lock,
           N_("Tells the running screensaver process to lock the screen immediately"), NULL },
-        { "cycle", 0, 0, G_OPTION_ARG_NONE, &do_cycle,
+        { "cycle", 'c', 0, G_OPTION_ARG_NONE, &do_cycle,
           N_("If the screensaver is active then switch to another graphics demo"), NULL },
-        { "activate", 0, 0, G_OPTION_ARG_NONE, &do_activate,
+        { "activate", 'a', 0, G_OPTION_ARG_NONE, &do_activate,
           N_("Turn the screensaver on (blank the screen)"), NULL },
-        { "deactivate", 0, 0, G_OPTION_ARG_NONE, &do_deactivate,
+        { "deactivate", 'd', 0, G_OPTION_ARG_NONE, &do_deactivate,
           N_("If the screensaver is active then deactivate it (un-blank the screen)"), NULL },
-        { "throttle", 0, 0, G_OPTION_ARG_NONE, &do_throttle,
-          N_("Disable running graphical themes while blanked"), NULL },
-        { "unthrottle", 0, 0, G_OPTION_ARG_NONE, &do_unthrottle,
-          N_("Enable running graphical themes while blanked (if applicable)"), NULL },
-        { "poke", 0, 0, G_OPTION_ARG_NONE, &do_poke,
+        { "poke", 'p', 0, G_OPTION_ARG_NONE, &do_poke,
           N_("Poke the running screensaver to simulate user activity"), NULL },
-        { "version", 0, 0, G_OPTION_ARG_NONE, &do_version,
+        { "inhibit", 'i', 0, G_OPTION_ARG_NONE, &do_inhibit,
+          N_("Inhibit the screensaver from activating.  Command blocks while inhibit is active."), NULL },
+        { "application-name", 'n', 0, G_OPTION_ARG_STRING, &inhibit_application,
+          N_("The calling application that is inhibiting the screensaver"), NULL },
+        { "reason", 'r', 0, G_OPTION_ARG_STRING, &inhibit_reason,
+          N_("The reason for inhibiting the screensaver"), NULL },
+        { "version", 'V', 0, G_OPTION_ARG_NONE, &do_version,
           N_("Version of this application"), NULL },
         { NULL }
 };
@@ -96,6 +100,46 @@ screensaver_is_running (DBusConnection *connection)
         return exists;
 }
 
+static DBusMessage *
+screensaver_send_message_inhibit (DBusConnection *connection,
+                                  const char     *application,
+                                  const char     *reason)
+{
+        DBusMessage    *message;
+        DBusMessage    *reply;
+        DBusError       error;
+	DBusMessageIter iter;
+
+        g_return_val_if_fail (connection != NULL, NULL);
+
+        dbus_error_init (&error);
+
+        message = dbus_message_new_method_call (GS_SERVICE, GS_PATH, GS_INTERFACE, "Inhibit");
+        if (message == NULL) {
+                g_warning ("Couldn't allocate the dbus message");
+                return NULL;
+        }
+
+	dbus_message_iter_init_append (message, &iter);
+	dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &application);
+	dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &reason);
+
+        reply = dbus_connection_send_with_reply_and_block (connection,
+                                                           message,
+                                                           -1, &error);
+        if (dbus_error_is_set (&error)) {
+                g_warning ("%s raised:\n %s\n\n", error.name, error.message);
+                reply = NULL;
+        }
+
+        dbus_connection_flush (connection);
+
+        dbus_message_unref (message);
+        dbus_error_free (&error);
+
+        return reply;
+}
+
 static DBusMessage *
 screensaver_send_message_bool (DBusConnection *connection,
                                const char     *name,
@@ -191,7 +235,7 @@ do_command (DBusConnection *connection)
                 DBusMessageIter iter;
                 dbus_bool_t     v;
 
-                reply = screensaver_send_message_void (connection, "getActive", TRUE);
+                reply = screensaver_send_message_void (connection, "GetActive", TRUE);
                 if (! reply) {
                         g_message ("Did not receive a reply from the screensaver.");
                         goto done;
@@ -213,11 +257,11 @@ do_command (DBusConnection *connection)
         }
 
         if (do_poke) {
-                reply = screensaver_send_message_void (connection, "Poke", FALSE);
+                reply = screensaver_send_message_void (connection, "SimulateUserActivity", FALSE);
         }
 
         if (do_activate) {
-                reply = screensaver_send_message_bool (connection, "setActive", TRUE);
+                reply = screensaver_send_message_bool (connection, "SetActive", TRUE);
                 if (! reply) {
                         g_message ("Did not receive a reply from the screensaver.");
                         goto done;
@@ -226,7 +270,7 @@ do_command (DBusConnection *connection)
         }
 
         if (do_deactivate) {
-                reply = screensaver_send_message_bool (connection, "setActive", FALSE);
+                reply = screensaver_send_message_bool (connection, "SetActive", FALSE);
                 if (! reply) {
                         g_message ("Did not receive a reply from the screensaver.");
                         goto done;
@@ -234,22 +278,17 @@ do_command (DBusConnection *connection)
                 dbus_message_unref (reply);
         }
 
-        if (do_throttle) {
-                reply = screensaver_send_message_bool (connection, "setThrottleEnabled", TRUE);
+        if (do_inhibit) {
+                reply = screensaver_send_message_inhibit (connection,
+                                                          inhibit_application ? inhibit_application : "Unknown",
+                                                          inhibit_reason ? inhibit_reason : "Unknown");
                 if (! reply) {
                         g_message ("Did not receive a reply from the screensaver.");
                         goto done;
                 }
                 dbus_message_unref (reply);
-        }
 
-        if (do_unthrottle) {
-                reply = screensaver_send_message_bool (connection, "setThrottleEnabled", FALSE);
-                if (! reply) {
-                        g_message ("Did not receive a reply from the screensaver.");
-                        goto done;
-                }
-                dbus_message_unref (reply);
+                return FALSE;
         }
 
  done:
@@ -263,7 +302,7 @@ main (int    argc,
       char **argv)
 {
         DBusConnection *connection;
-        DBusError       dbus_error;        
+        DBusError       dbus_error;
         GOptionContext *context;
         gboolean        retval;
         GError         *error = NULL;
@@ -272,9 +311,9 @@ main (int    argc,
         bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 # ifdef HAVE_BIND_TEXTDOMAIN_CODESET
         bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-# endif 
+# endif
         textdomain (GETTEXT_PACKAGE);
-#endif 
+#endif
 
         g_type_init ();
 
@@ -284,7 +323,6 @@ main (int    argc,
                 g_warning ("Locale not understood by C library, internationalization will not work\n");
 
         context = g_option_context_new (NULL);
-        g_option_context_set_ignore_unknown_options (context, TRUE);
         g_option_context_add_main_entries (context, entries, NULL);
         retval = g_option_context_parse (context, &argc, &argv, &error);
 
