@@ -34,6 +34,7 @@
 #include "gs-manager.h"
 #include "gs-watcher.h"
 #include "gs-fade.h"
+#include "gs-grab.h"
 
 #include "gs-listener-dbus.h"
 #include "gs-monitor.h"
@@ -53,16 +54,13 @@ struct GSMonitorPrivate
         GSManager      *manager;
         GSPrefs        *prefs;
         GSFade         *fade;
-};
+        GSGrab         *grab;
 
-enum {
-        PROP_0
+        guint           release_grab_id;
 };
 
 #define FADE_TIMEOUT 10000
 
-static GObjectClass *parent_class = NULL;
-
 G_DEFINE_TYPE (GSMonitor, gs_monitor, G_TYPE_OBJECT)
 
 static void
@@ -70,8 +68,6 @@ gs_monitor_class_init (GSMonitorClass *klass)
 {
         GObjectClass   *object_class = G_OBJECT_CLASS (klass);
 
-        parent_class = g_type_class_peek_parent (klass);
-
         object_class->finalize = gs_monitor_finalize;
 
         g_type_class_add_private (klass, sizeof (GSMonitorPrivate));
@@ -118,6 +114,20 @@ watcher_idle_cb (GSWatcher *watcher,
         return res;
 }
 
+static gboolean
+release_grab_timeout (GSMonitor *monitor)
+{
+        gboolean manager_active;
+
+        manager_active = gs_manager_get_active (monitor->priv->manager);
+        if (! manager_active) {
+                gs_grab_release (monitor->priv->grab);
+        }
+
+        monitor->priv->release_grab_id = 0;
+        return FALSE;
+}
+
 static gboolean
 watcher_idle_notice_cb (GSWatcher *watcher,
                         gboolean   in_effect,
@@ -137,7 +147,12 @@ watcher_idle_notice_cb (GSWatcher *watcher,
         if (in_effect) {
                 if (activation_enabled && ! inhibited) {
                         /* start slow fade */
-                        gs_fade_async (monitor->priv->fade, FADE_TIMEOUT, NULL, NULL);
+                        if (gs_grab_grab_offscreen (monitor->priv->grab, FALSE)) {
+                                gs_fade_async (monitor->priv->fade, FADE_TIMEOUT, NULL, NULL);
+                        } else {
+                                gs_debug ("Could not grab the keyboard so not performing idle warning fade-out");
+                        }
+
                         handled = TRUE;
                 }
         } else {
@@ -148,6 +163,12 @@ watcher_idle_notice_cb (GSWatcher *watcher,
                 if (! manager_active) {
                         gs_debug ("manager not active, performing fade cancellation");
                         gs_fade_reset (monitor->priv->fade);
+
+                        /* don't release the grab immediately to prevent typing passwords into windows */
+                        if (monitor->priv->release_grab_id != 0) {
+                                g_source_remove (monitor->priv->release_grab_id);
+                        }
+                        monitor->priv->release_grab_id = g_timeout_add (1000, (GSourceFunc)release_grab_timeout, monitor);
                 } else {
                         gs_debug ("manager active, skipping fade cancellation");
                 }
@@ -163,11 +184,17 @@ listener_lock_cb (GSListener *listener,
                   GSMonitor  *monitor)
 {
         gboolean res;
+        gboolean locked;
 
+        /* set lock flag before trying to activate screensaver
+           in case something tries to react to the ActiveChanged signal */
+
+        gs_manager_get_lock_active (monitor->priv->manager, &locked);
+        gs_manager_set_lock_active (monitor->priv->manager, TRUE);
         res = gs_listener_set_active (monitor->priv->listener, TRUE);
-        if (res) {
-                gs_manager_set_lock_active (monitor->priv->manager, TRUE);
-        } else {
+        if (! res) {
+                /* If we've failed then restore lock status */
+                gs_manager_set_lock_active (monitor->priv->manager, locked);
                 gs_debug ("Unable to lock the screen");
         }
 }
@@ -227,8 +254,8 @@ listener_throttle_changed_cb (GSListener *listener,
 }
 
 static void
-listener_poke_cb (GSListener *listener,
-                  GSMonitor  *monitor)
+listener_simulate_user_activity_cb (GSListener *listener,
+                                    GSMonitor  *monitor)
 {
         /* in case the screen isn't blanked reset the
            idle watcher */
@@ -289,7 +316,7 @@ disconnect_listener_signals (GSMonitor *monitor)
         g_signal_handlers_disconnect_by_func (monitor->priv->listener, listener_cycle_cb, monitor);
         g_signal_handlers_disconnect_by_func (monitor->priv->listener, listener_active_changed_cb, monitor);
         g_signal_handlers_disconnect_by_func (monitor->priv->listener, listener_throttle_changed_cb, monitor);
-        g_signal_handlers_disconnect_by_func (monitor->priv->listener, listener_poke_cb, monitor);
+        g_signal_handlers_disconnect_by_func (monitor->priv->listener, listener_simulate_user_activity_cb, monitor);
 }
 
 static void
@@ -305,8 +332,8 @@ connect_listener_signals (GSMonitor *monitor)
                           G_CALLBACK (listener_active_changed_cb), monitor);
         g_signal_connect (monitor->priv->listener, "throttle-changed",
                           G_CALLBACK (listener_throttle_changed_cb), monitor);
-        g_signal_connect (monitor->priv->listener, "poke",
-                          G_CALLBACK (listener_poke_cb), monitor);
+        g_signal_connect (monitor->priv->listener, "simulate-user-activity",
+                          G_CALLBACK (listener_simulate_user_activity_cb), monitor);
 }
 
 static void
@@ -373,6 +400,8 @@ gs_monitor_init (GSMonitor *monitor)
         connect_listener_signals (monitor);
 
         monitor->priv->fade = gs_fade_new ();
+        monitor->priv->grab = gs_grab_new ();
+
         monitor->priv->watcher = gs_watcher_new (monitor->priv->prefs->timeout);
         connect_watcher_signals (monitor);
 
@@ -400,12 +429,13 @@ gs_monitor_finalize (GObject *object)
         disconnect_prefs_signals (monitor);
 
         g_object_unref (monitor->priv->fade);
+        g_object_unref (monitor->priv->grab);
         g_object_unref (monitor->priv->watcher);
         g_object_unref (monitor->priv->listener);
         g_object_unref (monitor->priv->manager);
         g_object_unref (monitor->priv->prefs);
 
-        G_OBJECT_CLASS (parent_class)->finalize (object);
+        G_OBJECT_CLASS (gs_monitor_parent_class)->finalize (object);
 }
 
 GSMonitor *
