@@ -32,7 +32,6 @@
 #include <gtk/gtk.h>
 
 #include "gs-window.h"
-#include "gs-visual-gl.h"
 #include "subprocs.h"
 #include "gs-debug.h"
 
@@ -102,7 +101,6 @@ enum {
         PROP_MONITOR
 };
 
-static GObjectClass   *parent_class = NULL;
 static guint           signals [LAST_SIGNAL] = { 0, };
 
 G_DEFINE_TYPE (GSWindow, gs_window, GTK_TYPE_WINDOW)
@@ -162,6 +160,32 @@ gs_window_override_user_time (GSWindow *window)
         gdk_x11_window_set_user_time (GTK_WIDGET (window)->window, ev_time);
 }
 
+static void
+force_no_pixmap_background (GtkWidget *widget)
+{
+        static gboolean first_time = TRUE;
+
+        if (first_time) {
+                gtk_rc_parse_string ("\n"
+                                     "   style \"gs-theme-engine-style\"\n"
+                                     "   {\n"
+                                     "      bg_pixmap[NORMAL] = \"<none>\"\n"
+                                     "      bg_pixmap[INSENSITIVE] = \"<none>\"\n"
+                                     "      bg_pixmap[ACTIVE] = \"<none>\"\n"
+                                     "      bg_pixmap[PRELIGHT] = \"<none>\"\n"
+                                     "      bg[NORMAL] = { 0.0, 0.0, 0.0 }\n"
+                                     "      bg[INSENSITIVE] = { 0.0, 0.0, 0.0 }\n"
+                                     "      bg[ACTIVE] = { 0.0, 0.0, 0.0 }\n"
+                                     "      bg[PRELIGHT] = { 0.0, 0.0, 0.0 }\n"
+                                     "   }\n"
+                                     "   widget \"gs-window*\" style : highest \"gs-theme-engine-style\"\n"
+                                     "\n");
+                first_time = FALSE;
+        }
+
+        gtk_widget_set_name (widget, "gs-window");
+}
+
 static void
 clear_children (Window window)
 {
@@ -303,18 +327,90 @@ gs_window_real_unrealize (GtkWidget *widget)
                                               screen_size_changed,
                                               widget);
 
-        if (GTK_WIDGET_CLASS (parent_class)->unrealize) {
-                GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
+        if (GTK_WIDGET_CLASS (gs_window_parent_class)->unrealize) {
+                GTK_WIDGET_CLASS (gs_window_parent_class)->unrealize (widget);
+        }
+}
+
+static GdkVisual *
+get_best_visual (void)
+{
+        char         *command;
+        char         *std_output;
+        int           exit_status;
+        GError       *error;
+        unsigned long v;
+        char          c;
+        GdkVisual    *visual;
+
+        visual = NULL;
+
+        command = g_build_filename (LIBEXECDIR, "gnome-screensaver-gl-helper", NULL);
+
+        error = NULL;
+        g_spawn_command_line_sync (command,
+                                   &std_output,
+                                   NULL,
+                                   &exit_status,
+                                   &error);
+        g_free (command);
+
+        if (1 == sscanf (std_output, "0x%lx %c", &v, &c)) {
+                if (v != 0) {
+                        VisualID      visual_id;
+
+                        visual_id = (VisualID) v;
+                        visual = gdkx_visual_get (visual_id);
+
+                        gs_debug ("Found best visual for GL: 0x%x",
+                                  (unsigned int) visual_id);
+                }
+        }
+
+        g_free (std_output);
+
+        return visual;
+}
+
+static GdkColormap *
+get_best_colormap_for_screen (GdkScreen *screen)
+{
+        GdkColormap *colormap;
+        GdkVisual   *visual;
+
+        g_return_val_if_fail (screen != NULL, NULL);
+
+        visual = get_best_visual ();
+
+        colormap = NULL;
+        if (visual != NULL) {
+                colormap = gdk_colormap_new (visual, FALSE);
+        }
+
+        return colormap;
+}
+
+static void
+widget_set_best_colormap (GtkWidget *widget)
+{
+        GdkColormap *colormap;
+
+        g_return_if_fail (widget != NULL);
+
+        colormap = get_best_colormap_for_screen (gtk_widget_get_screen (widget));
+        if (colormap != NULL) {
+                gtk_widget_set_colormap (widget, colormap);
+                g_object_unref (colormap);
         }
 }
 
 static void
 gs_window_real_realize (GtkWidget *widget)
 {
-        gs_visual_gl_widget_set_best_colormap (widget);
+        widget_set_best_colormap (widget);
 
-        if (GTK_WIDGET_CLASS (parent_class)->realize) {
-                GTK_WIDGET_CLASS (parent_class)->realize (widget);
+        if (GTK_WIDGET_CLASS (gs_window_parent_class)->realize) {
+                GTK_WIDGET_CLASS (gs_window_parent_class)->realize (widget);
         }
 
         gs_window_override_user_time (GS_WINDOW (widget));
@@ -501,8 +597,8 @@ gs_window_real_show (GtkWidget *widget)
 {
         GSWindow *window;
 
-        if (GTK_WIDGET_CLASS (parent_class)->show) {
-                GTK_WIDGET_CLASS (parent_class)->show (widget);
+        if (GTK_WIDGET_CLASS (gs_window_parent_class)->show) {
+                GTK_WIDGET_CLASS (gs_window_parent_class)->show (widget);
         }
 
         gs_window_clear (GS_WINDOW (widget));
@@ -541,8 +637,8 @@ gs_window_real_hide (GtkWidget *widget)
 
         remove_watchdog_timer (window);
 
-        if (GTK_WIDGET_CLASS (parent_class)->hide) {
-                GTK_WIDGET_CLASS (parent_class)->hide (widget);
+        if (GTK_WIDGET_CLASS (gs_window_parent_class)->hide) {
+                GTK_WIDGET_CLASS (gs_window_parent_class)->hide (widget);
         }
 }
 
@@ -1272,8 +1368,8 @@ gs_window_real_key_press_event (GtkWidget   *widget,
                 queue_key_event (GS_WINDOW (widget), event);
         }
 
-        if (GTK_WIDGET_CLASS (parent_class)->key_press_event) {
-                GTK_WIDGET_CLASS (parent_class)->key_press_event (widget, event);
+        if (GTK_WIDGET_CLASS (gs_window_parent_class)->key_press_event) {
+                GTK_WIDGET_CLASS (gs_window_parent_class)->key_press_event (widget, event);
         }
 
         return TRUE;
@@ -1400,8 +1496,6 @@ gs_window_class_init (GSWindowClass *klass)
         GObjectClass   *object_class = G_OBJECT_CLASS (klass);
         GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
-        parent_class = g_type_class_peek_parent (klass);
-
         object_class->finalize     = gs_window_finalize;
         object_class->get_property = gs_window_get_property;
         object_class->set_property = gs_window_set_property;
@@ -1525,6 +1619,7 @@ gs_window_init (GSWindow *window)
                                | GDK_EXPOSURE_MASK
                                | GDK_ENTER_NOTIFY_MASK
                                | GDK_LEAVE_NOTIFY_MASK);
+        force_no_pixmap_background (GTK_WIDGET (window));
 }
 
 static void
@@ -1569,7 +1664,7 @@ gs_window_finalize (GObject *object)
         }
         gs_window_dialog_finish (window);
 
-        G_OBJECT_CLASS (parent_class)->finalize (object);
+        G_OBJECT_CLASS (gs_window_parent_class)->finalize (object);
 }
 
 GSWindow *
